package sun.nio.cs;

import java.util.Iterator;
import java.nio.charset.Charset;
import java.util.Map;
import java.nio.charset.spi.CharsetProvider;

public class FastCharsetProvider extends CharsetProvider
{
    private Map<String, String> classMap;
    private Map<String, String> aliasMap;
    private Map<String, Charset> cache;
    private String packagePrefix;
    
    protected FastCharsetProvider(String packagePrefix, Map<String, String> aliasMap, Map<String, String> classMap, Map<String, Charset> cache) {
        this.packagePrefix = packagePrefix;
        this.aliasMap = aliasMap;
        this.classMap = classMap;
        this.cache = cache;
    }
    
    private String canonicalize(String s) {
        String s2 = (String)this.aliasMap.get(s);
        return (s2 != null) ? s2 : s;
    }
    
    private static String toLower(String s) {
        int length = s.length();
        boolean n = true;
        int i = 0;
        while (i < length) {
            int char1 = s.charAt(i);
            if ((char1 - 65 | 90 - char1) >= 0) {
                n = false;
                break;
            }
            ++i;
        }
        if (!(n == 0)) {
            return s;
        }
        char[] value = new char[length];
        for (int j = 0; j < length; ++j) {
            int char2 = s.charAt(j);
            if ((char2 - 65 | 90 - char2) >= 0) {
                value[j] = (char)(char2 + 32);
            }
            else {
                value[j] = (char)char2;
            }
        }
        return new String(value);
    }
    
    private Charset lookup(String s) {
        String canonicalize = this.canonicalize(toLower(s));
        Charset charset = (Charset)this.cache.get(canonicalize);
        if (charset != null) {
            return charset;
        }
        String str = (String)this.classMap.get(canonicalize);
        if (str == null) {
            return null;
        }
        if (str.equals("US_ASCII")) {
            US_ASCII us_ASCII = new US_ASCII();
            this.cache.put(canonicalize, us_ASCII);
            return us_ASCII;
        }
        try {
            Charset charset2 = (Charset)Class.forName(this.packagePrefix + "." + str, true, this.getClass().getClassLoader()).newInstance();
            this.cache.put(canonicalize, charset2);
            return charset2;
        }
        catch (ClassNotFoundException | IllegalAccessException | InstantiationException ex) {
            return null;
        }
    }
    
    @Override
    public final Charset charsetForName(String s) {
        synchronized (this) {
            return this.lookup(this.canonicalize(s));
        }
    }
    
    @Override
    public final Iterator<Charset> charsets() {
        return new Iterator<Charset>() {
            Iterator<String> i = FastCharsetProvider.this.classMap.keySet().iterator();
            
            @Override
            public boolean hasNext() {
                return this.i.hasNext();
            }
            
            @Override
            public Charset next() {
                return FastCharsetProvider.access$100(FastCharsetProvider.this, (String)this.i.next());
            }
            
            @Override
            public void remove() {
                throw new UnsupportedOperationException();
            }
        };
    }
    
    static Charset access$100(FastCharsetProvider fastCharsetProvider, String s) {
        return fastCharsetProvider.lookup(s);
    }
}
